/********************************************************************************
*                                                                     
*       Copyright (C) 2008 Freescale Semiconductor, Inc.              
*       All Rights Reserved								              
*														              
* Filename:     main.c                
*														              
* Revision:      										              
*														              
* Functions:    Includes main function
*         
* Description:  
*
* Notes:        
*
*********************************************************************************/

#include <hidef.h> /* for EnableInterrupts macro */
#include "ADC16.h"
#include "derivative.h" /* include peripheral declarations */
#include "porting_support.h"
#include "ICS.h"
#include "interrupts.h"

/*********************************** Defines ***********************************/

#define BUSCLK_FREQ_KHZ		20000L	
#define TERMINAL_BAUD_RATE	9600 
#define SCIBD_BAUD_RATE ((BUSCLK_FREQ_KHZ * 10) / (16 * (TERMINAL_BAUD_RATE/100)))

#define ON  0xFF
#define OFF 0x00
#define SET 0xFF
#define CLEAR 0x00

/************************** Function Declarations ******************************/

void System_Init(void);
void WaitMicroSecondsInt(unsigned long);
void DataToPort_LSB_First(unsigned short);
void InitPortInterface(void);
unsigned short ADC_Conversion(unsigned char);

/*********************************** Function **********************************/
/********************************************************************
FUNCTION:		Init_Port_Interface
PURPOSE: 		Configure GPIOs for interface to PXI Hardware.
INPUTS NOTES:                
RETURNS NOTES: 
GENERAL NOTES:	Signal Descriptions:
				--------------------
				PTD[7:0] = ADC Conversion Data - low byte
				PTE[7:0] = ADC Conversion Data - high byte
				PTC5     = DUT Ready to Begin Next Ramp (Output)
				PTC4     = [PXI DAC Output] Ready for Next Conversion (Input)
				PTC2     = Data Valid trigger to HSDIO card. (Output)
********************************************************************/

void InitPortInterface(void) 
{
  
  // Configure 16 GPIOs for ADC conversion data transfer.
  PTDD = ON;   
  PTED = ON;           // Pre-configure data for high output.
  
  PTDDD = ON;
  PTEDD = ON;           // Configure Ports D and E as output.
  
  LCDC1 = 0x04;          // Full Complementary Drive Enable for Ports D and E.
  LCDSUPPLY = 0x03;      // Vsupply = %11 for FCD on ports D and E.
  LCDRVC = 0x08;         // Regulated Voltage disabled.
  
  // PTC2 is used as "Data Valid" trigger for HSDIO. 
  // PTC4 configured as an input for "Start Conversion" trigger.
  PTCDD = 0xef;         // PTC all output except PTE4.
  PTCD = 0x04;          // Initialize PTC2 output high.
  
} /* End Init_Port_Interface */


/********************************************************************
FUNCTION: 		ADC_SingleConversion
PURPOSE: 		Adjust ADC configuration per user specification and 
				collect a single conversion result on desired channel.
INPUTS NOTES: 	Channel - DF0 ASP ADC_IN[7:0].
RETURN NOTES:
GENERAL NOTES:  Returns 16-bit result. Bits [15:12] fixed to %0000.
********************************************************************/
unsigned short ADC_Conversion(unsigned char adc_chan)
{
	unsigned short ADC_result;

	// Test flow:
	// 	1. Clear previous flags.
	//  2. Start conversion on desired channel.
	//  3. Await completion (either via interrupt or flag polling).
	//  4. Read back 16-bit conversion value and pass to calling routine.
	
// STEP 1: Clear any previous COCO flag (clears when ADC channel written).
// STEP 2: Start a single ADC conversion on the desired channel.
  ADCSC1A_ADCHA = adc_chan;

// STEP 3: Await conversion completion COCO Flag.  
  while (ADCSC1A_COCOA == 0)
  {
  /* Wait for conversion to complete */
  }

// STEP 4: Read back the conversion result.

  // No explicit averaging necessary.  MCU can be configured to average internally.
	ADC_result = (ADCRA);
	
	return ADC_result;	 /* 16-bit data */
}



/********************************************************************
FUNCTION:		WaitMicroSecondsInt
PURPOSE: 		General purpose delay for interrupt routines.
INPUT NOTES:	32-bit Integer value in microseconds.
RETURN NOTES:	n/a
GENERAL NOTES:	4 CPU cycles per loop @ 8MHz (bus, per device init).
********************************************************************/
void WaitMicroSecondsInt (unsigned char delay_us)
{
	unsigned char j;
	
	delay_us = (delay_us * 2);  
	
	for(j=0; j<delay_us; j++);  // 4 cycles per For loop taken.
	
} // End WaitMicroSecondsInt

/********************************************************************
FUNCTION: 		DataToPort_LSB_First
PURPOSE: 		 Output ADC conversion data to GPIO.
INPUT NOTES:	16-bit data 
RETURN NOTES: 
GENERAL NOTES:  Output is LSB-first
********************************************************************/
void DataToPort_LSB_First (unsigned short adc_data)   
{                             

  PTDD = (adc_data & 0x00ff); // Write lower byte of conversion value to PTD.
  PTED = ((adc_data>>8) & 0x00ff); // Write upper byte of conversion value to PTE.
 
  PTCD_PTCD2 = 1;           // Assert "Data Valid" signal.
  WaitMicroSecondsInt(10);   
  PTCD_PTCD2 = 0;           // Deassert "Data Valid" after brief delay.
  WaitMicroSecondsInt(10); 
  
}	/* end of DataToPort_LSB_First */

/*******************************************************************************
* Function:        main
*
* Description:     
*
* Returns:         never return
*
* Notes:           
*                 
*******************************************************************************/

void main(void) {

  volatile unsigned short data;
  volatile unsigned short TEST;
  volatile unsigned short MODE;
  
  TEST = DIFF_SINGLE;   /* Select Test: DIFF_SINGLE or DIFF_DIFFERENTIAL */
  MODE = MODE_16;       /* Select Conversion Mode: MODE_16, MODE_12, MODE_10, MODE_8 */
  System_Init();        /* including pin mux control, ADC channel or I/O */
  
  DisableInterrupts; /* enable interrupts */
  
  /* blank interrupt routines in interrupts.c */ 
  /* please refer to table at top of file for ADC Channel -> ADC number correlataion */
  
  /* Configure ADC1 */
  
  ADC_Init((ADLPC_NORMAL|ADIV_8|ADLSMP_SHORT|MODE|ADICLK_BUS_2), (ADACKEN_ENABLED|ADHSC_HISPEED|ADLSTS_2) , 0, 0,
  (ADTRG_SW|ACFE_DISABLED|ACFGT_LESS|ACREN_DISABLED|REFSEL_EXT), (ADCO_SINGLE|AVGE_DISABLED), (AIEN_OFF|TEST|0x1F),
  (AIEN_OFF|TEST|0x1F));
    
  InitPortInterface();
  
    for(;;){
      ADCSC1A_ADCHA = 0x0B;       /* 0 & 4-11 */
    
    	do{
  	  /* Wait for conversion to complete */
    	}while (ADCSC2_ADACT == 1);	
      
      data = ADCRA;
      DataToPort_LSB_First(data);    // Write the conversion result out to GPIO.

    } /* loop forever */

} 

/******************************************************************************
Function Name	:	System_Init
Engineer      :	r54940
Date          :	12/09/08

Parameters		:	
Returns			  :	NONE
Notes         :	Configures system registers and I/O
******************************************************************************/
void System_Init(void)
{

  IRQSC = OFF;
  SPMSC1 = OFF;
  SPMSC2 = OFF;
  SPMSC3 = OFF;
  SOPT1 = 0x23;          /* Enable Stop Mode, Disable COP */
  SCGC1 = 0xFF;          
  SCGC2 = 0xFF;
  APCTL1 = 0xFF; 
  APCTL2 = 0xFF;
  
  /* disable all pull ups */
  PTAPE = ON;
  PTBPE = ON;
  PTCPE = ON;
  PTDPE = ON;
  PTEPE = ON;
  
  /* low drive strength */
  PTADS = OFF;
  PTBDS = OFF;
  PTCDS = OFF;
  PTDDS = OFF;
  PTEDS = OFF;
  
  /* disable slew rate */
  PTASE = OFF;
  PTBSE = OFF;
  PTCSE = OFF;
  PTDSE = OFF;
  PTESE = OFF;
  
  PTAD = CLEAR;
  PTBD = CLEAR;
  PTCD = CLEAR;
  PTDD = CLEAR;
  PTED = CLEAR;
   
  PTADD = SET;
  PTBDD = SET;
  PTCDD = SET;
  PTDDD = SET;
  PTEDD = SET;

}
